/*
 * Frame Diversion, Benoit Locher <Benoit.Locher@skf.com>
 *
 * Changes:
 * 		06/09/2000	BL:	initial version
 * 		15/10/2000	BL:	changed to a rule based structure
 */
 
#ifndef _LINUX_DIVERT_H
#define _LINUX_DIVERT_H

#include <linux/if_ether.h>

#ifdef __KERNEL__
#include <asm/spinlock.h>
#endif	/* __KERNEL__ */


struct sk_buff;
struct divert_rule;
typedef int (*process_rule_func)(struct sk_buff *, struct divert_rule *);


enum DIVERT_PROTO
{
	DIVERT_PROTO_IP=10,
	DIVERT_PROTO_TCP=11,
	DIVERT_PROTO_UDP=12,
	DIVERT_PROTO_ICMP=13,
	DIVERT_PROTO_ARP=20,
	DIVERT_PROTO_IPX=30,
};


typedef struct _divert_cidr
{
	unsigned long	ipnum,
					ipmask;
} divert_cidr;


typedef struct _divert_icmp_blk
{
	divert_cidr 	scidr;
	divert_cidr 	dcidr;
	unsigned short	type;
} divert_icmp_blk;


typedef struct _divert_tcp_blk
{
	divert_cidr 	scidr;
	divert_cidr 	dcidr;
	unsigned short	sport_start,
					sport_end,
					dport_start,
					dport_end;
} divert_tcp_blk;


typedef struct _divert_udp_blk
{
	divert_cidr 	scidr;
	divert_cidr 	dcidr;
	unsigned short	sport_start,
					sport_end,
					dport_start,
	 				dport_end;
} divert_udp_blk;


typedef struct _divert_ip_blk
{
	divert_cidr scidr;
	divert_cidr dcidr;
} divert_ip_blk;


typedef struct _divert_arp_blk
{

} divert_arp_blk;


typedef struct _divert_ipx_blk
{

} divert_ipx_blk;


typedef struct _divert_spx_blk
{

} divert_spx_blk;


typedef union _divert_proto
{
	divert_ip_blk	ip;
	divert_tcp_blk	tcp;
	divert_udp_blk	udp;
	divert_icmp_blk	icmp;
	divert_arp_blk	arp;
	divert_ipx_blk	ipx;
	divert_spx_blk	spx;
} divert_proto;


#ifdef __KERNEL__

struct divert_rule
{
	struct divert_rule	*prev,
						*next;
	unsigned long		proto_num;
	divert_proto		proto_blk;
	int					divert_locally;
	unsigned char		dest_mac[ETH_ALEN];
	process_rule_func	process_rule;
};


struct divert_blk
{
	int			divert;			/* diverter status on|off */
	rwlock_t	rwlock;			/* for rule list interprocess locking */
	struct divert_rule	*rule;	/* the rule doubly-linked list */
};

#endif	/* __KERNEL__ */

/*
 * Diversion control block, for configuration with the userspace tool
 * divert
 */

struct divert_cf
{
	int	cmd;							/* Command */
	unsigned int    rule_num;			/* Used to retrieve a specific rule */
	unsigned int    proto_num;			/* Protocol this rule works on */
	divert_proto    proto_blk;			/* Rule protocol description */
	int				status;				/* Used to get the status of the diverter */
	int				divert_locally;		/* acts locally to the specified interface */
	unsigned char	dest_mac[ETH_ALEN];	/* If not divert_locally, mac address it should
										   be forwarded to */
	char			dev_name[32];		/* Device name (eth0...) */
};


/* Diversion commands */
#define	DIVERT_SET_STATE		1	/* Switch diversion on or off */
#define	DIVERT_APPEND_RULE		2	/* Add a rule */
#define	DIVERT_INSERT_RULE		3	/* Insert a rule */
#define	DIVERT_REPLACE_RULE		4	/* Replace a rule */
#define	DIVERT_DELETE_RULE		5	/* Delete a rule */
#define	DIVERT_FLUSH_RULES		6	/* Flush all the rules */
#define	DIVERT_GET_RULE			7	/* Retrieve a rule */
#define	DIVERT_GET_STATUS		8	/* retrieve the version + diverter status */


#ifdef __KERNEL__

/* diverter functions */
#include <linux/skbuff.h>
void   dv_init(void);
int    alloc_divert_blk(struct device *);
void   free_divert_blk(struct device *);
int    divert_ioctl(unsigned int, struct divert_cf *);
void   divert_frame(struct sk_buff *);

/* Rule processing routines */
int    process_ip_rule(struct sk_buff *, struct divert_rule *);
int    process_ip_tcp_rule(struct sk_buff *, struct divert_rule *);
int    process_ip_udp_rule(struct sk_buff *, struct divert_rule *);
int    process_ip_icmp_rule(struct sk_buff *, struct divert_rule *);

/* Rule manipulation routines */
struct divert_rule *divert_alloc_rule(void);
void   divert_free_rule(struct divert_rule *);
struct divert_rule *divert_get_rule(struct device *, int);
int    divert_append_rule(struct device *, struct divert_rule *);
int    divert_insert_rule(struct device *, struct divert_rule *, int);
int    divert_delete_rule(struct device *, int);
int    divert_replace_rule(struct device *, struct divert_rule *, int);
int    divert_flush_rules(struct device *);

#endif	/*  __KERNEL__ */

#endif	/* _LINUX_DIVERT_H */
